/*
 * Copyright (C) 2008 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.oem.battery;

import android.annotation.SdkConstant;
import android.annotation.SdkConstant.SdkConstantType;
import android.content.Context;
import android.os.Binder;
import android.os.IBinder;
import android.os.Handler;
import android.os.HandlerThread;
import android.os.Looper;
import android.os.Message;
import android.os.RemoteException;
import android.os.WorkSource;
import android.os.Messenger;
import android.util.Log;
import android.util.SparseArray;

import java.util.concurrent.CountDownLatch;

import com.android.internal.util.AsyncChannel;
import com.android.internal.util.Protocol;

import java.util.List;

import android.os.ServiceManager;
import android.os.BatteryManager;
/**
 * This class provides the primary API for managing all aspects of Wi-Fi
 * connectivity. Get an instance of this class by calling
 * {@link android.content.Context#getSystemService(String) Context.getSystemService(Context.WIFI_SERVICE)}.

 * It deals with several categories of items:
 * <ul>
 * <li>The list of configured networks. The list can be viewed and updated,
 * and attributes of individual entries can be modified.</li>
 * <li>The currently active Wi-Fi network, if any. Connectivity can be
 * established or torn down, and dynamic information about the state of
 * the network can be queried.</li>
 * <li>Results of access point scans, containing enough information to
 * make decisions about what access point to connect to.</li>
 * <li>It defines the names of various Intent actions that are broadcast
 * upon any sort of change in Wi-Fi state.
 * </ul>
 * This is the API to use when performing Wi-Fi specific operations. To
 * perform operations that pertain to network connectivity at an abstract
 * level, use {@link android.net.ConnectivityManager}.
 */
public class BatteryInterfaceManager {

    private static final String TAG = "BatteryInterfaceManager";
	
	/*stdcommand */
	private static final String BQ27541_REG_CNTL = "0000" ;
	private static final String BQ27541_REG_AR	 = "0200" ;
	private static final String BQ27541_REG_ARTTE = "0400";
	private static final String BQ27541_REG_TEMP = "0600";
	private static final String BQ27541_REG_VOLT = "0800";
	private static final String BQ27541_REG_FLAGS = "0A00";
	private static final String BQ27541_REG_NAC	 = "0C00";
	private static final String BQ27541_REG_FAC	 = "0e00";
	private static final String BQ27541_REG_RM	 = "1000";
	private static final String BQ27541_REG_FCC	 = "1200";
	private static final String BQ27541_REG_AI	 = "1400";
	private static final String BQ27541_REG_TTE	 = "1600";
	private static final String BQ27541_REG_TTF	 = "1800";
	private static final String BQ27541_REG_SI	 = "1a00";
	private static final String BQ27541_REG_STTE = "1c00";
	private static final String BQ27541_REG_MLI	 = "1e00";
	private static final String BQ27541_REG_MLTTE = "2000";
	private static final String BQ27541_REG_AE	 = "2200";
	private static final String BQ27541_REG_AP	 = "2400";
	private static final String BQ27541_REG_TTECP = "2600";
	private static final String BQ27541_REG_SOH	 = "2800";
	private static final String BQ27541_REG_CC	 = "2a00";
	private static final String BQ27541_REG_SOC	 = "2c00";
	private static final String BQ27541_REG_NIC	 = "2e00";
	private static final String BQ27541_REG_ICR	 = "3000";
	private static final String BQ27541_REG_LOGIDX = "3200";
	private static final String BQ27541_REG_LOGBUF = "3400";
	private static final String BQ27541_REG_DC		= "3c00";
	/*subcommand */
	private static final String BQ27541_SUBCMD_CTNL_STATUS  = "0000";
	public static final String BQ27541_SUBCMD_DEVCIE_TYPE  = "0001";
	public  static final String BQ27541_SUBCMD_FW_VER  = "0002";
	public static final String BQ27541_SUBCMD_HW_VER  = "0003";
	private static final String BQ27541_SUBCMD_DF_CSUM  = "0004";
	private static final String BQ27541_SUBCMD_PREV_MACW   = "0007";
	public static final String BQ27541_SUBCMD_CHEM_ID   = "0008";
	private static final String BQ27541_SUBCMD_BD_OFFSET   = "0009";
	private static final String BQ27541_SUBCMD_INT_OFFSET  = "000a";
	private static final String BQ27541_SUBCMD_CC_VER   = "000b";
	private static final String BQ27541_SUBCMD_OCV  = "000c";
	private static final String BQ27541_SUBCMD_BAT_INS   = "000d";
	private static final String BQ27541_SUBCMD_BAT_REM   = "000e";
	private static final String BQ27541_SUBCMD_SET_HIB   = "0011";
	private static final String BQ27541_SUBCMD_CLR_HIB   = "0012";
	private static final String BQ27541_SUBCMD_SET_SLP   = "0013";
	private static final String BQ27541_SUBCMD_CLR_SLP   = "0014";
	private static final String BQ27541_SUBCMD_FCT_RES   = "0015";
	private static final String BQ27541_SUBCMD_ENABLE_DLOG  = "0018";
	private static final String BQ27541_SUBCMD_DISABLE_DLOG = "0019";
	private static final String BQ27541_SUBCMD_SEALED   = "0020";
	private static final String BQ27541_SUBCMD_ENABLE_IT    = "0021";
	private static final String BQ27541_SUBCMD_DISABLE_IT   = "0023";
	private static final String BQ27541_SUBCMD_CAL_MODE  = "0040";
	private static final String BQ27541_SUBCMD_RESET   = "0041";
	
	
	/* extended commands*/
	private static final String BQ27541_EXTCMD_SERIALNO  = "4000";  //0x40-0x4C   
	private static final String BQ27541_EXTCMD_FIRST_FULL		= "4d00";
	private static final String BQ27541_EXTCMD_MANUF_DATE  = "5000";  //0x40-0x4C   
	   

	/* backup battery commands*/

	
    /** @hide */
    protected static IBatteryManager battery;
    
    
    private static final BatteryInterfaceManager sInstance = new BatteryInterfaceManager();
	    
    public static BatteryInterfaceManager getDefault() {
        return sInstance;
    }

    private BatteryInterfaceManager() {
    }
    
    private IBatteryManager getBatteryManagerService() {
        return IBatteryManager.Stub.asInterface(ServiceManager.getService("Ibattery"));
    }

	
	
	private int BatteryControl(String command){
		try {
			return getBatteryManagerService().BatteryControl(command);
		} catch (RemoteException ex) {
			return -1;
		} catch (NullPointerException ex) {
			return -1;
		}
		
	}
	
	private int GetBatteryInfoint(String command){
		try {
			return getBatteryManagerService().GetBatteryInfoint(command);
		} catch (RemoteException ex) {
			return -1;
		} catch (NullPointerException ex) {
			return -1;
		}
		
	}

	private String GetBatteryInfoString(String command){
		try {
			return getBatteryManagerService().GetBatteryInfoString(command);
		} catch (RemoteException ex) {
			return "NULL";
		} catch (NullPointerException ex) {
			return "NULL";
		}
		
	}	

	public int getControl(String subcmd){
	
		return BatteryControl(subcmd);
	}
	
	public int getAtRate(){
	
		return BatteryControl(BQ27541_REG_AR);
	}
	
	public int getAtRateTimeToEmpty(){
	
		return BatteryControl(BQ27541_REG_ARTTE);
	}
	
	public int getTemperature(){
	
		return GetBatteryInfoint(BQ27541_REG_TEMP);
	}
	
	public int getVoltage(){
	
		return BatteryControl(BQ27541_REG_VOLT);
	}
	
	public int getFlags(){
	
		return BatteryControl(BQ27541_REG_FLAGS);
	}
	
	public int getNominalAvailableCapacity(){
	
		return BatteryControl(BQ27541_REG_NAC);
	}
	
	public int getFullAvailableCapacity(){
	
		return BatteryControl(BQ27541_REG_FAC);
	}

	public int getRemainingCapacity(){
	
		return BatteryControl(BQ27541_REG_RM);
	}

	public int getFullChargeCapacity(){
	
		return BatteryControl(BQ27541_REG_FCC);
	}	

	public int getAverageCurrent(){
	
		return BatteryControl(BQ27541_REG_AI);
	}		

	public int getTimeToEmpty(){
	
		return BatteryControl(BQ27541_REG_TTE);
	}	

	public int getTimeToFull(){
	
		return BatteryControl(BQ27541_REG_TTF);
	}	
	
	public int getStandbyCurrent(){
	
		return BatteryControl(BQ27541_REG_SI);
	}			

	public int getStandbyTimeToEmpty(){
	
		return BatteryControl(BQ27541_REG_STTE);
	}			
				
	public int getMaxLoadCurrent(){
	
		return BatteryControl(BQ27541_REG_MLI);
	}	

	public int getMaxLoadTimeToEmpty(){
	
		return BatteryControl(BQ27541_REG_MLTTE);
	}

	public int getAvailableEnergy(){
	
		return BatteryControl(BQ27541_REG_AE);
	}

	public int getAveragePower(){
	
		return BatteryControl(BQ27541_REG_AP);
	}			

	public int getTTEatConstantPower(){
	
		return BatteryControl(BQ27541_REG_TTECP);
	}	

	public int getCycleCount(){
	
		return BatteryControl(BQ27541_REG_CC);
	}		

	public int getStateOfCharge(){
	
		return BatteryControl(BQ27541_REG_SOC);
	}


	public int getDesignCapacity(){
	
		return GetBatteryInfoint(BQ27541_REG_DC);
	}


	public int getFirstFullyChargedCapacity(){
	
		return GetBatteryInfoint(BQ27541_EXTCMD_FIRST_FULL);
	}


	public String getBatterySerialNumber(){
	
		return GetBatteryInfoString(BQ27541_EXTCMD_SERIALNO);
	}


	public String getManufacturingDate(){
	
		return GetBatteryInfoString(BQ27541_EXTCMD_MANUF_DATE);
	}	
		
	public int getBackupBatteryType(){
	
		return GetBatteryInfoint(BatteryManager.EXTRA_TYPE);
	}
	
	public int getBackupBatteryCapacity(){
	
		return GetBatteryInfoint(BatteryManager.EXTRA_BACKUP_BATTERY_CAPACITY);
	}
	
	public int getBackupBatteryMvoltage(){
	
		return GetBatteryInfoint(BatteryManager.EXTRA_BACKUP_BATTERY_MVOLTAGE);
	}
	
	public int getBackupBatteryChargeState(){
	
		return GetBatteryInfoint(BatteryManager.EXTRA_BACKUP_BATTERY_CHARGE_STATE);
	}
}
